@@ -1,3 +1,5 @@
+'''20210904
+'''
 import os
 import sys
 from collections import defaultdict
@@ -5,18 +7,18 @@ import PyQt5.QtWidgets as qtw
 import PyQt5.QtCore as qtc
 import PyQt5.QtGui as qtg
 from PyQt5.QtMultimedia import QMediaPlayer, QMediaContent
-
+import numpy as np
 import ctypes
-#修正setWindowIcon显示在任务栏的问题
+# 修正setWindowIcon显示在任务栏的问题
 ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID("myappid")
-#修正ui在不同分辨率屏幕上的尺寸问题
+# 修正ui在不同分辨率屏幕上的尺寸问题
 qtc.QCoreApplication.setAttribute(qtc.Qt.AA_EnableHighDpiScaling)
 app = qtw.QApplication(sys.argv)
 LOOP_STARTED = False
 
-#窗口显示栈，当前显示窗口为栈的第一个窗口
+# 窗口显示栈，当前显示窗口为栈的第一个窗口
 WINDOW_STACK = []
-#广播机制，里面的数据结构是
+# 广播机制，里面的数据结构是
 # {"msgid1":{"message":"tom","handlers":[func1,func2]},
 #  "msgid2":{"message":"[1,2,3,4,5]","handlers":[func3,func4]}}
 MESSAGE_HANDLERS = {}
@@ -52,9 +54,64 @@ class MsgMode:
 class CodeQTSingals(qtc.QObject):
     changeWindow = qtc.pyqtSignal(qtw.QMainWindow)
     messageSingal = qtc.pyqtSignal(dict)
+    int_signal = qtc.pyqtSignal(int)
+    float_signal = qtc.pyqtSignal(float)
+    str_signal = qtc.pyqtSignal(str)
+    list_signal = qtc.pyqtSignal(list)
+    dict_signal = qtc.pyqtSignal(dict)
+    empty_signal = qtc.pyqtSignal()
+    ndarray_signal = qtc.pyqtSignal(np.ndarray)
+
+
+###############################################
+#
+# 2021年8月20日新增UIWidget
+# 自定义QWidget用来“膨胀”自定义组件视图
+# 为QWidget组件
+#
+###############################################
+
+
+class UIWidget(qtw.QWidget):
+
+    def __init__(self, widget_ui, objname="Form"):
+        super().__init__(parent=None)
+        # 根据objname的不同
+        # 动态的执行代码为self.ui赋值
+        exec(f"self.ui=widget_ui.Ui_{objname}()")
+        self.ui.setupUi(self)
+
+    def get_widget(self, obj_name):
+        '''
+        根据组件的objectName获取对应的组件对象
+        '''
+        if hasattr(self.ui, obj_name):
+            return getattr(self.ui, obj_name)
+        else:
+            raise SystemExit(f"没有objectName为{obj_name}的组件")
+
+    def set_image(self, obj_name, path):
+        if hasattr(self.ui, obj_name):
+            obj = getattr(self.ui, obj_name)
+            obj.setStyleSheet(f"border-image:url({path})")
+
+    def get_text(self, id):
+        wi = self.get_widget(id)
+        if wi and hasattr(wi, "text"):
+            return wi.text()
+        else:
+            raise SystemExit("无法获取该组件文本内容")
+
+    def set_text(self, obj_name, text):
+        w = self.get_widget(obj_name)
+        if hasattr(w, 'setText'):
+            w.setText(text)
+        else:
+            raise SystemExit('该组件无法设置文本')
 
 
 class Window(qtw.QMainWindow):
+
     def __init__(self, myui):
         super().__init__(parent=None)
         if hasattr(myui, 'Ui_Form'):
@@ -78,6 +135,34 @@ class Window(qtw.QMainWindow):
         self.media_player = QMediaPlayer(self)
         self.media_player.stateChanged.connect(
             self.do_mediaplayer_statechanged)
+        # 2021年8月21日03:40新增
+        self.show_exec_flag = False
+        # 2021年8月21日03:40新增
+        # 看一下start_thread是否执行完毕
+        self.is_thread_runnging = False
+
+        self.tasks = []
+
+    # 2021年8月21日4点19分新增方法
+    # 在一个线程中执行func函数
+    def start_thread(self, func, *args, **kwargs):
+        this = self
+
+        class MyThread(qtc.QThread):
+
+            def __init__(self) -> None:
+                super().__init__()
+
+            def run(self) -> None:
+                this.is_thread_runnging = True
+                func(*args, **kwargs)
+                this.is_thread_runnging = False
+
+        # self.thread = MyThread()
+        t = MyThread()
+        self.tasks.append(t)
+        t.start()
+        # self.thread.start()
 
     def do_mediaplayer_statechanged(self, state):
         if state == QMediaPlayer.PlayingState:
@@ -100,7 +185,7 @@ class Window(qtw.QMainWindow):
         a0: 执行过installEventFilter，也就是委托Window代理事件的组件对象
         e: e是在a0上发生的事件对象(QEvent)
         重载eventFilter并不会处理a0上发生的所有事件，只处理通过watch_widget为a0"注册"的事件类型
-        
+
         '''
         if a0 in self.watchedwidgets:
             if e.type() in self.watchedwidgets[a0]:
@@ -144,7 +229,7 @@ class Window(qtw.QMainWindow):
     def remove_window_siganl(self, signal, func):
         self.mysignals[signal].remove(func)
 
-    #等待子类改写
+    # 等待子类改写
     def setup(self):
         pass
 
@@ -216,8 +301,8 @@ class Window(qtw.QMainWindow):
         '''
         def myfunc(event):
 
-            func(*args, **kwargs)  #执行自定义函数功能
-            #执行完自定义函数后，调用QLabel默认函数处理后续QLabel默认功能
+            func(*args, **kwargs)  # 执行自定义函数功能
+            # 执行完自定义函数后，调用QLabel默认函数处理后续QLabel默认功能
             return qtw.QLabel.mousePressEvent(self.get_widget(obj_name), event)
 
         wi = self.get_widget(obj_name)
@@ -225,10 +310,10 @@ class Window(qtw.QMainWindow):
         wi.mousePressEvent = myfunc
 
     def remove_click_button(self, widget):
-        #如果传入的是QPushButton组件对象
+        # 如果传入的是QPushButton组件对象
         if isinstance(widget, qtw.QWidget):
             widget.clicked.disconnect()
-        #如果传入的是QPushButton组件的ID
+        # 如果传入的是QPushButton组件的ID
         elif isinstance(widget, str):
             self.remove_click_button(self.get_widget(widget))
 
@@ -237,10 +322,10 @@ class Window(qtw.QMainWindow):
         将func作为单击QPushButton时产生的clicked信号的槽函数
         来处理单击事件
         '''
-        #如果传入的是QPushButton组件对象，则直接为对象绑定clicked信号的槽函数
+        # 如果传入的是QPushButton组件对象，则直接为对象绑定clicked信号的槽函数
         if isinstance(widgt, qtw.QWidget):
             widgt.clicked.connect(func)
-        #如果传入的是QPushButton组件的ID，则获取ID对应的组建后再绑定clicked的槽函数
+        # 如果传入的是QPushButton组件的ID，则获取ID对应的组建后再绑定clicked的槽函数
         elif isinstance(widgt, str):
             self.on_click_button(self.get_widget(widgt), func)
 
@@ -255,12 +340,13 @@ class Window(qtw.QMainWindow):
             buttons = type[2]
         return func(self, title, msg, buttons)
 
-    def play_media(self, filename):
+    def play_media(self, filename, vol=100):
         abs_path = os.path.abspath(filename)
         url = qtc.QUrl.fromLocalFile(abs_path)
         c = QMediaContent(url)
         self.media_player.setMedia(c)
         self.media_player.play()
+        self.media_player.setVolume(vol)
 
     def set_echomode(self, obj_name, mode):
         '''
@@ -283,6 +369,7 @@ class Window(qtw.QMainWindow):
         win = self
 
         class Canvas(qtw.QWidget):
+
             def __init__(self, parent) -> None:
                 super().__init__(parent=parent)
                 self.lastpoint = qtc.QPoint()
@@ -336,15 +423,58 @@ class Window(qtw.QMainWindow):
         canvas.setGeometry(qtc.QRect(x0, y0, w, h))
         return canvas
 
-    def draw_image(self, imagename, x0, y0, w=0, h=0):
+    def draw_text(self,
+                  x,
+                  y,
+                  content,
+                  font="微软雅黑",
+                  size=14,
+                  color="black",
+                  width=100,
+                  height=100,
+                  angle=0):
+        '''
+        20210901新增方法，可以在窗口书写文字
+        '''
+        class My_Inner_Widget(qtw.QWidget):
+
+            def __init__(self, parent) -> None:
+                super().__init__(parent=parent)
+                self.font = qtg.QFont(font, size)
+
+            def paintEvent(self, a0: qtg.QPaintEvent):
+                p = qtg.QPainter(self)
+                if type(color) == "str":
+                    p.setPen(qtg.QColor(color))
+                else:
+                    r, g, b = color
+                    p.setPen(qtg.QColor(r, g, b))
+                p.rotate(angle)
+                p.setFont(self.font)
+                p.drawText(a0.rect(), qtc.Qt.AlignLeft, content)
+
+        mywidget = My_Inner_Widget(self)
+        mywidget.setGeometry(qtc.QRect(x, y, width, height))
+        return mywidget
+
+    def draw_image(self, imagename, x0, y0, w=0, h=0, angle=0):
         '''
         创建自定义控件Widget，通过重写其paintEvent
         该自定义Widget外形是在paintEvent中绘制的图像
+        20210901新增参数角度angle，默认值0
         '''
         class MyImage(qtw.QWidget):
+
             def __init__(self, parent) -> None:
                 super().__init__(parent=parent)
+                self.change_x = 0
+                self.change_y = 0
                 self.img = qtg.QPixmap(imagename)
+                # 使用qtg.QTransform对象来旋转图片
+                transform = qtg.QTransform().rotate(angle)
+                self.img = self.img.transformed(transform,
+                                                qtc.Qt.SmoothTransformation)
+
                 if w and h:
                     self.imgw = w
                     self.imgh = h
@@ -352,6 +482,55 @@ class Window(qtw.QMainWindow):
                     self.imgw = self.img.width()
                     self.imgh = self.img.height()
 
+                self.img.scaled(self.imgw, self.imgh)
+
+                self.dir_x = 1  # 向右
+                self.dir_y = 1  # 向下
+
+            def move(self, x=10, y=10):
+                self.move_x(x)
+                self.move_y(y)
+
+            def move_x(self, dist_x=10):
+
+                i = dist_x / abs(dist_x)
+                if self.dir_x == i and self.change_x == 0:
+                    # 初始方向与要前进的方向一致
+                    self.change_x = dist_x
+                if self.dir_x == -i and self.change_x == 0:
+                    # 初始方向与要前进的方向不一致
+                    self.change_x = abs(dist_x)
+                    self.dir_x = -1
+                    self.flip()
+                pos_x = self.x() + self.dir_x * self.change_x
+                self.setGeometry(
+                    qtc.QRect(pos_x, self.y(), self.width(), self.height()))
+
+            def move_y(self, dist_y=10):
+                i = dist_y / abs(dist_y)
+                if self.dir_y == i and self.change_y == 0:
+                    # 初始方向与要前进的方向一致
+                    self.change_y = dist_y
+                if self.dir_y == -i and self.change_y == 0:
+                    # 初始方向与要前进的方向不一致
+                    self.change_y = abs(dist_y)
+                    self.dir_y = -1
+                pos_y = self.y() + self.dir_y * self.change_y
+                self.setGeometry(
+                    qtc.QRect(self.x(), pos_y, self.width(), self.height()))
+
+            def turnover_x(self):
+                self.dir_x *= -1
+
+            def turnover_y(self):
+                self.dir_y *= -1
+
+            def flip(self):
+                transform = qtg.QTransform().rotate(180, qtc.Qt.YAxis)
+                self.img = self.img.transformed(transform,
+                                                qtc.Qt.SmoothTransformation)
+                self.repaint()
+
             def paintEvent(self, a0: qtg.QPaintEvent):
                 p = qtg.QPainter(self)
                 p.drawPixmap(0, 0, self.imgw, self.imgh, self.img)
@@ -360,6 +539,19 @@ class Window(qtw.QMainWindow):
         mywidget.setGeometry(qtc.QRect(x0, y0, mywidget.imgw, mywidget.imgh))
         return mywidget
 
+    def set_piximage(self, objname, imgname, angle=0):
+        '''20210902新增方法，为组件添加pixmap前景图
+        图像会根据控件的大小被缩放
+        '''
+        img = qtg.QPixmap(imgname)
+        # 使用qtg.QTransform对象来旋转图片
+        transform = qtg.QTransform().rotate(angle)
+        img = img.transformed(transform, qtc.Qt.SmoothTransformation)
+        mywidget = self.get_widget(objname)
+        if hasattr(mywidget, "setScaledContents"):
+            mywidget.setScaledContents(True)
+        mywidget.setPixmap(img)
+
     def add_widget(self, WidgetClazz, objname, x0, y0, w, h):
         '''
         动态添加Widget
@@ -399,6 +591,9 @@ class Window(qtw.QMainWindow):
         self.codeqt_signals.changeWindow.connect(w.show)
         self.codeqt_signals.changeWindow.emit(self)
         self.hide()
+        # 2021年8月21日03:40新增
+        self.show_exec_flag = False
+        ##########################
         self.codeqt_signals.changeWindow.disconnect(w.show)
 
     def back_window(self):
@@ -407,6 +602,9 @@ class Window(qtw.QMainWindow):
         如果当前窗口是从A窗口跳转过来的，则该函数会跳转会B窗口
         '''
         if WINDOW_STACK[0] == self:
+            # 2021年8月21日03:40新增
+            self.show_exec_flag = False
+            ##########################
             WINDOW_STACK.pop(0).hide()
             WINDOW_STACK[0].show()
 
@@ -437,11 +635,22 @@ class Window(qtw.QMainWindow):
         让窗口显示
         '''
         self.setup()
-        #只要show函数获得执行，就意味着self将被显示出来
-        #凡是当前显示窗口就是WINDOW_STACK最上方的窗口
+        # 只要show函数获得执行，就意味着self将被显示出来
+        # 凡是当前显示窗口就是WINDOW_STACK最上方的窗口
         WINDOW_STACK.insert(0, self)
+        # 2021年8月21日03:40新增
+        self.show_exec_flag = True
         return super().show()
 
+    ##################
+    # 2021年8月21日03:38新增
+    # 判定窗口是否执行了show()指令
+    #
+    ##################
+
+    def is_show(self):
+        return self.show_exec_flag
+
     def set_image(self, obj_name, path):
         if hasattr(self.ui, obj_name):
             obj = getattr(self.ui, obj_name)
@@ -469,8 +678,15 @@ class Window(qtw.QMainWindow):
                 )
 
 
+def get_image_size(pathname):
+    '''获得图形的尺寸信息：宽度，高度
+    '''
+    img = qtg.QImage(pathname)
+    return img.size().width(), img.size().height()
+
+
 def run():
     global LOOP_STARTED
     if not LOOP_STARTED:
         LOOP_STARTED = True
-        sys.exit(app.exec())
\ No newline at end of file
+        sys.exit(app.exec())
